/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/dm/somtoolbox/license.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.somtoolbox.apps.viewer.controls;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import at.tuwien.ifs.somtoolbox.SOMToolboxException;
import at.tuwien.ifs.somtoolbox.apps.viewer.CommonSOMViewerStateData;
import at.tuwien.ifs.somtoolbox.apps.viewer.SOMPane;
import at.tuwien.ifs.somtoolbox.apps.viewer.fileutils.ExportUtils;
import at.tuwien.ifs.somtoolbox.util.FileUtils;
import at.tuwien.ifs.somtoolbox.util.GridBagConstraintsIFS;

/**
 * Displays the class legend, i.e. the names of the different classes and the colours attached to them. Class colours
 * can be changed; the assignment can also be save and loaded to/from a file.
 * 
 * @author Michael Dittenbach
 * @author Rudolf Mayer
 * @author Thomas Lidy
 * @version $Id: ClassLegendPane.java 3873 2010-10-28 09:29:58Z frank $
 */
public class ClassLegendPane extends AbstractViewerControl {

    private static final long serialVersionUID = 1L;

    private static final String[] columnNames = new String[] { "Class Name", "Color" };

    private ColourLegendTable table = null;

    private JLabel noClassesLoaded = null;

    private SOMPane mapPane = null;

    private JFileChooser fc;

    public ClassLegendPane(SOMPane mapPane, String title, CommonSOMViewerStateData state) {
        super(title, state, new GridBagLayout());
        this.mapPane = mapPane;

        if (mapPane.getClassLegendNames() != null) {
            // this.colors = state.inputDataObjects.getClassInfo().getClassColors();
            initClassTable();
            setVisible(true);
        } else {
            initNoClassInfo();
            setVisible(false);
        }
    }

    public void initNoClassInfo() {
        if (table != null) {
            remove(table);
        }
        if (noClassesLoaded != null) {
            remove(noClassesLoaded);
        } else {
            noClassesLoaded = new JLabel("No class information file was loaded!");
        }
        add(noClassesLoaded);
        repaint();
    }

    public void initClassTable() {
        GridBagConstraintsIFS b = new GridBagConstraintsIFS(GridBagConstraints.NORTHWEST, GridBagConstraints.CENTER).setWeights(
                1, 1);

        if (table != null) {
            remove(table);
        }
        if (noClassesLoaded != null) {
            remove(noClassesLoaded);
        }
        table = new ColourLegendTable(mapPane.getClassLegendNames(), mapPane.getClassLegendColors(), columnNames,
                new ClassSelectionHandler());
        // Jakob: to propagate color-changes
        table.getModel().addTableModelListener(new TableModelListener() {
            @Override
            public void tableChanged(TableModelEvent e) {
                TableModel tm = table.getModel();
                for (int i = e.getFirstRow(); i <= e.getLastRow(); i++) {
                    if (i == TableModelEvent.HEADER_ROW) {
                        continue;
                    }
                    Color c = (Color) tm.getValueAt(i, 1);
                    state.inputDataObjects.getClassInfo().setClassColor(i, c);
                    state.mapPNode.setClassColor(i, c);
                }
            }
        });

        addMouseListener(new MouseAdapter() {
            @Override
            public void mouseReleased(MouseEvent e) {
                table.clearSelection();
            }
        });
        JScrollPane sp = new JScrollPane(table);
        // we need to fix the size of the JScrollPane, as otherwise by default, it has requested way too much height.
        sp.setPreferredSize(new Dimension(state.controlElementsWidth, table.getPreferredHeight()));
        add(sp, b.setGridWidth(3));

        // init load/save buttons
        JButton btLoadColors = new JButton("Load Colours");
        btLoadColors.setMargin(new Insets(2, 2, 1, 2));
        btLoadColors.setFont(smallFont);
        btLoadColors.addActionListener(new ActionListener() {
            // listener for loading
            @Override
            public void actionPerformed(ActionEvent e) {
                initFileChooser();
                fc.showOpenDialog(ClassLegendPane.this);
                File file = fc.getSelectedFile();
                if (file == null) {
                    return;
                }

                if (state.inputDataObjects.getClassInfo().loadClassColours(file)) {
                    // then update visualization
                    updateClassColours();
                    mapPane.repaint();
                }
            }
        });
        add(btLoadColors, b.nextRow().setGridWidth(1).resetWeights());

        JButton btSaveColors = new JButton("Save Colours");
        btSaveColors.setFont(smallFont);
        btSaveColors.setMargin(new Insets(2, 2, 1, 2));
        btSaveColors.addActionListener(new ActionListener() {
            // listener for saving
            @Override
            public void actionPerformed(ActionEvent e) {
                if (table.getColors() != null) {
                    initFileChooser();
                    fc.showSaveDialog(ClassLegendPane.this);
                    File file = fc.getSelectedFile();
                    if (file == null) {
                        return;
                    }

                    Logger.getLogger("at.tuwien.ifs.somtoolbox").info("Saving colors to file: " + file.toString());

                    try {
                        FileWriter fw = new FileWriter(file);
                        for (int i = 0; i < table.getColors().length; i++) {
                            fw.write(table.getColors()[i].getRed() + " " + table.getColors()[i].getGreen() + " "
                                    + table.getColors()[i].getBlue() + "\n");
                        }
                        fw.close();
                    } catch (IOException ex) {
                        Logger.getLogger("at.tuwien.ifs.somtoolbox").warning(
                                "Could not write colors to file! " + ex.getMessage());
                        ex.printStackTrace();
                    }
                }
            }
        });
        add(btSaveColors, b.nextCol());

        JButton btExportlegend = new JButton("Export Legend");
        btExportlegend.setFont(smallFont);
        btExportlegend.setMargin(new Insets(2, 2, 1, 2));
        btExportlegend.addActionListener(new ActionListener() {
            // listener for exporting the class legend to an image file
            @Override
            public void actionPerformed(ActionEvent e) {
                File file = ExportUtils.getFilePath(ClassLegendPane.this, state.getFileChooser(),
                        "Export class legend to");
                if (file != null) {
                    try {
                        FileUtils.saveImageToFile(file.getAbsolutePath(), table.asBufferedImage());
                    } catch (SOMToolboxException e1) {
                        Logger.getLogger("at.tuwien.ifs.somtoolbox").severe(
                                "Could not write class legend to file '" + file.getAbsolutePath() + "': "
                                        + e1.getMessage());
                    }
                }
            }
        });
        add(btExportlegend, b.nextCol());

    }

    private void initFileChooser() {
        if (fc == null) {
            fc = new JFileChooser(state.fileChooser.getCurrentDirectory());
        }
    }

    public void updateClassColours() {
        table.setColors(state.inputDataObjects.getClassInfo().getClassColors());
        for (int i = 0; i < table.getRowCount(); i++) {
            table.setValueAt(table.getColors()[i], i, 1);
        }
    }

    private class ClassSelectionHandler implements ListSelectionListener {
        @Override
        public void valueChanged(ListSelectionEvent e) {
            ListSelectionModel lsm = (ListSelectionModel) e.getSource();

            // System.out.println("value changed");
            Vector<Integer> selIndices = new Vector<Integer>();
            if (e.getValueIsAdjusting() == false) {
                if (lsm.isSelectionEmpty()) {
                    // no selection
                    mapPane.updateClassSelection(null);
                } else {
                    // Find out which indexes are selected.
                    int minIndex = lsm.getMinSelectionIndex();
                    int maxIndex = lsm.getMaxSelectionIndex();
                    for (int i = minIndex; i <= maxIndex; i++) {
                        if (lsm.isSelectedIndex(i)) {
                            selIndices.addElement(new Integer(i));
                        }
                    }
                    int[] selectedIndices = new int[selIndices.size()];
                    for (int i = 0; i < selectedIndices.length; i++) {
                        selectedIndices[i] = selIndices.elementAt(i).intValue();
                    }
                    mapPane.updateClassSelection(selectedIndices);
                }
            }
        }
    }

    @Override
    public void setEnabled(boolean enabled) {
        table.setEnabled(enabled);
    }

    public Color[] getColors() {
        return table.getColors();
    }

    /* (non-Javadoc)
     * @see at.tuwien.ifs.somtoolbox.apps.viewer.controls.AbstractViewerControl#isFullFunctional()
     */
    @Override
    public boolean isFullFunctional() {
        return mapPane.getClassLegendNames() != null;
    }

}
